
# q-expansion of newform 1050.2.d.c, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1050
weight = 2
poly_data = [1, 0, 3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[1, 1, 1, 0], 1], [[0, 2, 0, 1], 1], [[-1, 1, -1, 0], 1]]

hecke_ring_character_values = [[701, [-1, 0, 0, 0]], [127, [-1, 0, 0, 0]], [451, [-1, 0, 0, 0]]]
aps_data = [[-1, 0, 0, 0], [0, 0, 1, 1], [0, 0, 0, 0], [-1, 1, 2, 0], [0, 2, 2, 2], [0, -1, 0, 1], [0, 1, 4, 1], [0, -2, -6, -2], [-4, 0, 0, 0], [0, 3, 4, 3], [0, 1, -4, 1], [0, -1, 2, -1], [0, -2, 0, 2], [0, 4, 0, 4], [0, -2, -4, -2], [-2, 2, 0, -2], [-2, -2, 0, 2], [0, 1, 6, 1], [0, 0, -12, 0], [0, -1, -6, -1], [-10, -1, 0, 1], [4, 4, 0, -4], [0, -7, -6, -7], [8, -2, 0, 2], [-2, 1, 0, -1], [-10, -2, 0, 2], [-6, -7, 0, 7], [-4, -6, 0, 6], [-2, -2, 0, 2], [2, -4, 0, 4], [0, 3, -4, 3], [-2, 6, 0, -6], [-6, 2, 0, -2], [0, -2, 14, -2], [0, -3, 8, -3], [16, 4, 0, -4], [8, 1, 0, -1], [0, 2, 8, 2], [0, -4, 4, -4], [0, 6, 2, 6], [0, 4, 10, 4], [0, -5, -10, -5], [0, 1, 6, 1], [0, 0, 6, 0], [-6, 2, 0, -2], [0, 5, 16, 5], [-8, 0, 0, 0], [14, 3, 0, -3], [0, -1, 2, -1], [0, -1, 10, -1], [14, -2, 0, 2], [0, -5, 6, -5], [0, 8, 8, 8], [-10, -2, 0, 2], [0, 5, 8, 5], [-8, -4, 0, 4], [-2, -2, 0, 2], [0, 3, 28, 3], [0, -9, -6, -9], [0, 0, 20, 0], [12, 1, 0, -1], [0, -6, 10, -6], [12, 5, 0, -5], [20, -2, 0, 2], [-2, -3, 0, 3], [6, 4, 0, -4], [8, -4, 0, 4], [0, 6, 14, 6], [0, -2, 0, 2], [0, -5, -14, -5], [0, 3, 24, 3], [0, 5, 6, 5], [2, 5, 0, -5], [0, 5, 26, 5], [8, 8, 0, -8], [0, 8, 4, 8], [0, -5, -24, -5], [-12, -9, 0, 9], [0, 2, 12, 2], [0, -6, 0, -6], [-22, -12, 0, 12], [-6, 2, 0, -2], [0, -11, -26, -11], [18, 7, 0, -7], [0, -5, -4, -5], [-16, -2, 0, 2], [0, 2, 8, 2], [0, 4, 22, 4], [-2, -4, 0, 4], [0, 5, 16, 5], [0, -3, -30, -3], [-28, -8, 0, 8], [0, 11, 4, 11], [0, 6, 6, 6], [-8, -8, 0, 8], [0, 10, 16, 10], [-18, 2, 0, -2], [-32, -4, 0, 4], [-24, -15, 0, 15], [26, 4, 0, -4], [0, -6, -28, -6], [10, -2, 0, 2], [0, -5, -34, -5], [0, 2, -12, 2], [4, 12, 0, -12], [2, 5, 0, -5], [0, 3, -14, 3], [0, 17, 8, 17], [0, 1, -38, 1], [0, -2, 8, -2], [18, 1, 0, -1], [0, -13, -22, -13], [2, 0, 0, 0], [0, 0, -14, 0], [4, -8, 0, 8], [0, 6, 16, 6], [-16, 3, 0, -3], [0, 12, 20, 12], [14, 8, 0, -8], [0, -2, 34, -2], [0, -3, 22, -3], [0, 2, -22, 2], [0, 0, 18, 0], [36, 0, 0, 0], [0, 4, -26, 4], [0, -13, -28, -13], [18, -2, 0, 2], [32, 2, 0, -2], [-30, 3, 0, -3], [4, 13, 0, -13], [-4, -4, 0, 4], [8, 2, 0, -2], [16, -6, 0, 6], [0, -7, 6, -7], [4, -8, 0, 8], [0, 0, 36, 0], [0, 6, -18, 6], [-40, 3, 0, -3], [0, 6, 14, 6], [0, -10, -24, -10], [0, 12, 22, 12], [0, -3, -28, -3], [0, 5, 16, 5], [24, 12, 0, -12], [0, 5, -14, 5], [-8, 2, 0, -2], [4, 3, 0, -3], [0, 7, -20, 7], [0, -4, 22, -4], [28, 2, 0, -2], [0, 7, 30, 7], [8, 6, 0, -6], [0, 8, 4, 8], [0, 2, -20, 2], [0, 2, -20, 2], [0, 7, 42, 7], [20, 20, 0, -20], [-28, -8, 0, 8], [-26, -13, 0, 13], [22, 0, 0, 0], [-48, -10, 0, 10], [30, -6, 0, 6], [0, -19, -16, -19], [-58, 0, 0, 0], [6, -6, 0, 6], [0, -6, 20, -6], [20, -2, 0, 2], [-24, -1, 0, 1], [-22, -12, 0, 12], [0, -10, 6, -10], [0, -4, -18, -4], [0, 11, 6, 11], [0, -7, 18, -7], [0, 0, 26, 0], [0, 1, -28, 1], [-12, -2, 0, 2], [8, 16, 0, -16], [0, 11, 16, 11], [26, 5, 0, -5], [0, 23, 14, 23], [0, 7, 0, 7], [2, -10, 0, 10], [0, -3, -42, -3], [0, -15, -32, -15], [-24, -10, 0, 10], [58, -2, 0, 2], [0, 19, 22, 19], [-16, -7, 0, 7], [-38, 2, 0, -2], [-4, 4, 0, -4], [-6, 13, 0, -13], [0, -16, 0, 16], [12, 20, 0, -20], [2, -20, 0, 20], [28, 16, 0, -16], [0, -11, 0, -11], [-18, -20, 0, 20], [0, 11, 22, 11], [0, -11, -8, -11], [0, -10, 16, -10], [0, 15, -4, 15], [0, 7, -28, 7], [8, -19, 0, 19], [0, 10, 36, 10], [-22, -2, 0, 2], [0, -6, -38, -6], [0, -11, 20, -11], [-16, -12, 0, 12], [0, 12, 48, 12], [0, -2, 38, -2], [0, 4, 22, 4], [-18, 10, 0, -10], [0, -23, -32, -23], [0, -11, 22, -11], [-4, 6, 0, -6], [0, -8, -8, -8], [0, -9, -16, -9], [-28, -20, 0, 20], [0, -12, 0, 12], [18, 22, 0, -22], [-42, -14, 0, 14], [0, -11, -20, -11], [0, 2, 28, 2], [0, -1, 40, -1], [0, 3, 26, 3], [18, -2, 0, 2], [0, -5, 16, -5], [0, 19, 10, 19], [-6, 7, 0, -7], [0, 6, -34, 6], [0, -3, -22, -3], [0, -10, -14, -10], [-12, 6, 0, -6], [0, 0, -20, 0], [-32, -13, 0, 13], [0, 0, 48, 0], [0, -2, -16, -2], [26, 30, 0, -30], [0, -4, -38, -4], [28, 26, 0, -26], [-40, 4, 0, -4], [0, -22, -22, -22], [-14, -5, 0, 5], [2, 12, 0, -12], [0, 3, -16, 3], [20, 20, 0, -20], [-34, -1, 0, 1], [-22, -4, 0, 4], [-8, -8, 0, 8], [36, 10, 0, -10], [0, 17, -10, 17], [0, 8, 0, -8], [-4, -6, 0, 6], [0, -20, -4, -20], [0, -12, 14, -12], [0, 10, 46, 10], [-18, -10, 0, 10], [44, 7, 0, -7], [0, 12, 50, 12], [26, -1, 0, 1], [0, -7, 44, -7], [0, -22, 0, 22], [0, 5, -34, 5], [4, -17, 0, 17], [0, -7, -4, -7], [0, -10, -34, -10], [0, 7, 28, 7], [12, 20, 0, -20], [0, -4, -68, -4], [-30, -16, 0, 16], [0, -13, -18, -13], [0, -8, 0, -8], [30, 19, 0, -19], [24, -6, 0, 6], [14, -13, 0, 13], [66, 5, 0, -5], [-16, 12, 0, -12], [-2, -2, 0, 2], [22, -10, 0, 10], [2, -20, 0, 20], [0, 12, 48, 12], [20, -2, 0, 2], [0, 10, 28, 10], [0, 23, 38, 23], [20, 23, 0, -23], [0, -3, 2, -3], [0, 6, 22, 6], [42, 20, 0, -20], [0, -9, -8, -9], [24, 24, 0, -24], [0, -19, -4, -19], [0, -1, 22, -1], [-18, -24, 0, 24], [34, -8, 0, 8], [0, -5, -54, -5], [-46, -16, 0, 16], [0, 1, 16, 1], [0, -4, 22, -4], [66, 16, 0, -16], [-20, 3, 0, -3], [18, 27, 0, -27], [62, 0, 0, 0], [-12, -2, 0, 2]]
